(function(){var e,t;e=this,t=function(e){"use strict";e.buildURL=function buildURL(e={}){let t=e.baseURL||"";const n=e.queryData;if(n){let e;"string"==typeof n?(e=n,0===e.indexOf("?")&&(e=e.substr(1))):e=$.param(n),e&&(t+=`?${e}`)}const o=e.anchor;return o&&(0===o.indexOf("#")?t+=o:t+=`#${o}`),t},e.parseQueryString=function parseQueryString(e,t={}){const n=t.allowMultiValue;let o,s=e.indexOf("?");o=-1===s?e:e.substr(s+1);const r={};if(0===o.length)return r;const i=o.split("&");for(let e=0;e<i.length;e++){const t=i[e];let o,l;s=t.indexOf("="),-1===s?(o=decodeURIComponent(t),l=null):(o=decodeURIComponent(t.substr(0,s)),l=decodeURIComponent(t.substr(s+1))),n&&r.hasOwnProperty(o)?_.isArray(r[o])?r[o].push(l):r[o]=[r[o],l]:r[o]=l}return r}},"object"==typeof exports&&"undefined"!=typeof module?t(exports):"function"==typeof define&&define.amd?define(["exports"],t):t((e="undefined"!=typeof globalThis?globalThis:e||self).Djblets=e.Djblets||{}),Promise.withResolver=function withResolver(){let e;return[new Promise((t,n)=>{e=t}),e]}}).call(this);
